# -*- coding: utf-8 -*-
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import gridspec

time_act = 0.5 * 0.85 # Timings module
tau_rise = 0.25 * 0.55 * time_act
LDCC = 4
Lsi0 = 1.51
l_si = 2
Lsi_start = 2.3
Lsi_end = 0.5
Lsi_end_LDCC = 0.5
SfAct = 120
vMax = 7

ylim_C = [0, 3.3]

t = np.linspace(0, 0.5, 5001)
Ti = t / tau_rise
ft1 = Ti * Ti * Ti * np.exp(-Ti) * (8.0 - Ti) * (8.0 - Ti) * 0.02 / tau_rise



plt.figure(1, clear=True, figsize=(12, 8))

gs = gridspec.GridSpec(4, 4, height_ratios=[1, 3, 1, 1])

# %% Constant l_si
ax_Lsi = plt.subplot(gs[0, 0])
ax_ft1 = plt.subplot(4, 4, 5)
ax_C = plt.subplot(4, 4, 9)
ax_Sf = plt.subplot(4, 4, 13)

ax_ft1.plot(t, ft1, c='k')
ax_Lsi.plot(t, t*0+Lsi_start, c='k')

C = np.cumsum(ft1, axis=0) * t[1]
L = Lsi_start / Lsi0 - 1
ax_C.plot(t, C, '-', c=[0, 0, 0], zorder=99)
ax_Sf.plot(t, 1.51 * C * L * SfAct, 'k', zorder=99)

ax_ft1.spines[['right', 'top']].set_visible(False)
ax_Lsi.spines[['right', 'top']].set_visible(False)
ax_C.spines[['right', 'top']].set_visible(False)
ax_Sf.spines[['right', 'top']].set_visible(False)

ax_C.set_ylim(ylim_C)

for Lsi_end_var in np.linspace(1.3*Lsi_start - 0.3*Lsi0 , Lsi0, 5):
    l_si = np.linspace(Lsi_end_var, Lsi_end_var, len(t))
    l_si[l_si<Lsi0] = Lsi0
    tanh = np.tanh(LDCC * (l_si - Lsi0)**2)
    ax_ft1.plot(t, ft1 * tanh, '--', c=[0.5, 0.5, 0.5])
    ax_Lsi.plot(t, l_si, '--', c=[0.5, 0.5, 0.5])

    C = np.cumsum(ft1 * tanh, axis=0) * t[1]
    L = l_si / Lsi0 - 1
    LsiDot = (l_si[1]-l_si[0])/t[1]
    LNormSe = LsiDot/vMax + 1

    ax_C.plot(t, C, '--', c=[0.5, 0.5, 0.5])
    ax_Sf.plot(t, 1.51 * C * L * SfAct * LNormSe, '--', c=[0.5, 0.5, 0.5])


ax_Sf.set_xlabel('Time [s]')
ax_ft1.set_ylabel(r'$\frac{dC}{dt}$ [$s^-1$]')
ax_Lsi.set_ylabel(r'l_si [$\mu$m]')
ax_C.set_ylabel(r'$C_{up}$')
ax_Sf.set_ylabel(r'$\sigma_{f,act}$ [kPa] ' + '\n' + r' $ = C \cdot (l_{si} - l_{si,0}) \cdot S_{f,act}$')
ax_Lsi.set_title('Constant Sarcomere Length')
Lsi_ylim = ax_Lsi.get_ylim()

for ax in [ax_Sf, ax_ft1, ax_Lsi, ax_C]:
    ax.yaxis.set_label_coords(-0.2, 0.5)

# %% Decreasing l_si
ax_Lsi = plt.subplot(4, 4, 2)
ax_ft1 = plt.subplot(4, 4, 6)
ax_C = plt.subplot(4, 4, 10)
ax_Sf = plt.subplot(4, 4, 14)
ax_ft1.plot(t, ft1, c='k')
ax_Lsi.plot(t, t*0+Lsi_start, c='k')
C = np.cumsum(ft1, axis=0) * t[1]
ax_C.plot(t, C, '-', c=[0, 0, 0], zorder=99)
L = Lsi_start / Lsi0 - 1
ax_Sf.plot(t, 1.51 * C * L * SfAct, 'k', zorder=99)

ax_ft1.spines[['right', 'top']].set_visible(False)
ax_Lsi.spines[['right', 'top']].set_visible(False)
ax_C.spines[['right', 'top']].set_visible(False)
ax_Sf.spines[['right', 'top']].set_visible(False)

ax_C.set_ylim(ylim_C)
ax_Lsi.set_ylim(Lsi_ylim)


for Lsi_end_var in np.linspace(2.5, -2.5, 6)[1:]:
    l_si = np.linspace(Lsi_start, Lsi_end_var, len(t))
    l_si[l_si<Lsi0] = Lsi0
    tanh = np.tanh(LDCC * (l_si - Lsi0)**2)
    ax_ft1.plot(t, ft1 * tanh, '--', c=[0.5, 0.5, 0.5])
    ax_Lsi.plot(t, l_si, '--', c=[0.5, 0.5, 0.5])

    C = np.cumsum(ft1 * tanh, axis=0) * t[1]
    L = l_si / Lsi0 - 1
    LsiDot = (l_si[1]-l_si[0])/t[1]
    LNormSe = LsiDot/vMax + 1

    ax_C.plot(t, C, '--', c=[0.5, 0.5, 0.5])
    ax_Sf.plot(t, 1.51 * C * L * SfAct * LNormSe, '--', c=[0.5, 0.5, 0.5])

ax_Sf.set_xlabel('Time [s]')
ax_Lsi.set_title('Varying shortening rate')
Lsi_ylim = ax_Lsi.get_ylim()

# %% Different LDCC
ax_Lsi = plt.subplot(4, 4, 3)
ax_ft1 = plt.subplot(4, 4, 7)
ax_C = plt.subplot(4, 4, 11)
ax_Sf = plt.subplot(4, 4, 15)
ax_ft1.plot(t, ft1, c='k')
ax_Lsi.plot(t, t*0+Lsi_start, c='k')
C = np.cumsum(ft1, axis=0) * t[1]
ax_C.plot(t, C, '-', c=[0, 0, 0], zorder=99)
L = Lsi_start / Lsi0 - 1
ax_Sf.plot(t, 1.51 * C * L * SfAct, 'k', zorder=99)


ax_ft1.spines[['right', 'top']].set_visible(False)
ax_Lsi.spines[['right', 'top']].set_visible(False)
ax_C.spines[['right', 'top']].set_visible(False)
ax_Sf.spines[['right', 'top']].set_visible(False)

ax_C.set_ylim(ylim_C)

for LDCC_var in [0.5, 1, 2, 4, 8, 16, 64, 1e99]:
    l_si = np.linspace(Lsi_start, Lsi_end_LDCC, len(t))
    l_si[l_si<Lsi0] = Lsi0
    tanh = np.tanh(LDCC_var * (l_si - Lsi0)**2)
    ax_ft1.plot(t, ft1 * tanh, '--', c=[0.5, 0.5, 0.5])
    ax_Lsi.plot(t, l_si, '--', c=[0.5, 0.5, 0.5])

    C = np.cumsum(ft1 * tanh, axis=0) * t[1]
    L = l_si / Lsi0 - 1
    LsiDot = (l_si[1]-l_si[0])/t[1]
    LNormSe = LsiDot/vMax + 1


    ax_C.plot(t, C, '--', c=[0.5, 0.5, 0.5])
    ax_Sf.plot(t, 1.51 * C * L * SfAct * LNormSe, '--', c=[0.5, 0.5, 0.5])
ax_Sf.set_xlabel('Time [s]')
ax_Lsi.set_title('Varying LDCC')
ax_Lsi.set_ylim(Lsi_ylim)

# %% Different TauRise
ax_Lsi = plt.subplot(4, 4, 4)
ax_ft1 = plt.subplot(4, 4, 8)
ax_C = plt.subplot(4, 4, 12)
ax_Sf = plt.subplot(4, 4, 16)
ax_ft1.plot(t, ft1, c='k', zorder=99)
ax_Lsi.plot(t, t*0+Lsi_start, c='k', zorder=99)
C = np.cumsum(ft1, axis=0) * t[1]
ax_C.plot(t, C, '-', c=[0, 0, 0], zorder=99)
L = Lsi_start / Lsi0 - 1
ax_Sf.plot(t, 1.51 * C * L * SfAct, 'k', zorder=99)



ax_ft1.spines[['right', 'top']].set_visible(False)
ax_Lsi.spines[['right', 'top']].set_visible(False)
ax_C.spines[['right', 'top']].set_visible(False)
ax_Sf.spines[['right', 'top']].set_visible(False)

ax_C.set_ylim(ylim_C)

for tau_rise_var in (np.linspace(0.15, 0.35, 3) * 0.55 * time_act):
    Ti = t / tau_rise_var
    ft1 = Ti * Ti * Ti * np.exp(-Ti) * (8.0 - Ti) * (8.0 - Ti) * 0.02 / tau_rise

    l_si = np.linspace(Lsi_start, Lsi_start, len(t))
    l_si[l_si<Lsi0] = Lsi0
    tanh = np.tanh(LDCC * (l_si - Lsi0)**2)
    ax_ft1.plot(t, ft1 * tanh, '-', c=[0.5, 0.5, 0.5])
    ax_Lsi.plot(t, l_si, '-', c=[0.5, 0.5, 0.5])
    ax_C.plot(t, np.cumsum(ft1 * tanh, axis=0) * t[1], '-', c=[0.5, 0.5, 0.5])

    l_si = np.linspace(Lsi_start, Lsi_end, len(t))
    l_si[l_si<Lsi0] = Lsi0
    tanh = np.tanh(LDCC * (l_si - Lsi0)**2)
    ax_ft1.plot(t, ft1 * tanh, '--', c=[0.5, 0.5, 0.5])
    ax_Lsi.plot(t, l_si, '--', c=[0.5, 0.5, 0.5])

    C = np.cumsum(ft1 * tanh, axis=0) * t[1]
    L = l_si / Lsi0 - 1
    LsiDot = (l_si[1]-l_si[0])/t[1]
    LNormSe = LsiDot/vMax + 1


    ax_C.plot(t, C, '--', c=[0.5, 0.5, 0.5])
    ax_Sf.plot(t, 1.51 * C * L * SfAct * LNormSe, '--', c=[0.5, 0.5, 0.5])

ax_Sf.set_xlabel('Time [s]')
ax_Lsi.set_title(r'$\tau_{rise} = 0.55 \cdot T_R \cdot t_{act}$')
ax_Lsi.set_ylim(Lsi_ylim)

plt.suptitle('How the contraction (positive upslope C-curve) depends on length, LDCC, and TauRise\n' +
             r'$\frac{dC_{up}}{dt} = F_{rise}(t, \tau_{rise}) \cdot \tanh(LDCC \cdot (l_{si} - l_{si,0})^2)$', fontsize=16, fontweight='bold')

plt.tight_layout()

plt.draw()
